﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;

public abstract class SphereState {

	protected SphereController _controller;

	public virtual SphereStateNames StateName {
		get {
			return SphereStateNames.NullStateID;
		}
	}

	public SphereState(SphereController caller) {
		_controller = caller;
	}

	public virtual void OnBegin(SphereStateNames previous) {
	}

	public virtual void OnEnd(SphereStateNames next) {
	}

	public virtual void MainUpdate() {
	}

	public virtual void ReceiveInput(float xAxis, float yAxis) {
	}
}

public class SphereStateStopped : SphereState {

	public override SphereStateNames StateName {
		get {
			return SphereStateNames.Stopped;
		}
	}

	public SphereStateStopped(SphereController caller) : base(caller) {
	}

	public override void OnBegin(SphereStateNames previous) {
		base.OnBegin(previous);

		_controller.ChangeSphereColor(Color.red);
	}
}

public class SphereStateFourDir : SphereState {

	private float _speed = 4f;

	public override SphereStateNames StateName {
		get {
			return SphereStateNames.FourDir;
		}
	}

	public SphereStateFourDir(SphereController caller) : base(caller) {
	}

	public override void OnBegin(SphereStateNames previous) {
		base.OnBegin(previous);

		_controller.ChangeSphereColor(Color.green);
	}

	public override void ReceiveInput(float xAxis, float yAxis) {
		float rotation = _controller.Sphere.transform.rotation.eulerAngles.z;

		if (xAxis > 0) {
			rotation = 0f;
		} else if (xAxis < 0) {
			rotation = 180f;
		}

		if (yAxis > 0) {
			rotation = 90f;
		} else if (yAxis < 0) {
			rotation = 270f;
		}

		_controller.SetRotationSphere(Quaternion.Euler(0f, 0f, rotation));

		_controller.TranslateSphere(xAxis * _speed * Time.deltaTime, yAxis * _speed * Time.deltaTime, Space.World);
	}
}

public class SphereStateCar : SphereState {

	private float _accel = 5f;
	private float _maxVel = 5f;

	private float _velocity = 0f;

	public override SphereStateNames StateName {
		get {
			return SphereStateNames.Car;
		}
	}

	public SphereStateCar(SphereController caller) : base(caller) {
	}

	public override void OnBegin(SphereStateNames previous) {
		base.OnBegin(previous);

		_velocity = 0f;

		_controller.ChangeSphereColor(Color.blue);
	}

	public override void ReceiveInput(float xAxis, float yAxis) {
		Quaternion newFacing = _controller.Sphere.transform.rotation;

		newFacing = Quaternion.Euler(0f, 0f, newFacing.eulerAngles.z - xAxis * 90f * Time.deltaTime);

		_controller.SetRotationSphere(newFacing);

		if (yAxis != 0) {
			_velocity += _accel * Time.deltaTime;
			Mathf.Clamp(_velocity, -_maxVel, _maxVel);
		} else {
			if (_velocity < 0f) {
				_velocity += _maxVel * 2f * Time.deltaTime;

				Mathf.Clamp(_velocity, -_maxVel, 0f);
			} else if (_velocity > 0f) {
				_velocity -= _maxVel * 2f * Time.deltaTime;

				Mathf.Clamp(_velocity, 0f, _maxVel);
			}
		}

		_controller.TranslateSphere(yAxis * _velocity * Time.deltaTime, 0f);
	}
}